<?php

/**
 * @file
 * Advanced aggregation bundler module.
 */

/**
 * Implements hook_advagg_aggregate_grouping_alter().
 */
function advagg_bundler_advagg_aggregate_grouping_alter(&$groups, $type) {
  // If bundler is disabled then do not do any more processing.
  if (!advagg_bundler_enabled($type)) {
    return;
  }

  $config = \Drupal::config('advagg_bundler.settings');
  $max = $config->get('max_' . $type);
  $modifiable = [];
  $files = 0;
  foreach ($groups as $key => $group) {
    if (isset($group['type'], $group['preprocess']) && $group['type'] == 'file' && $group['preprocess']) {
      $modifiable[$key] = $group;
      $modifiable[$key]['order'] = $key;
      $modifiable[$key]['file_count'] = count($group['items']);
      $files += count($group['items']);
    }
  }

  // If more bundles than $max return. $groups is already set to least
  // possible number of groups with current sort. Enabling sort external first
  // may help decrease number of bundles.
  if (count($modifiable) > $max || !$modifiable) {
    return;
  }

  $target_files = ceil($files / $max);
  $final_groups = [];
  $bundles = 1;
  foreach ($groups as $key => $group) {
    if (!isset($modifiable[$key]) || $bundles == $max) {
      $final_groups[] = $group;
      continue;
    }
    $splits = round($modifiable[$key]['file_count'] / $target_files);
    if ($splits < 2) {
      $final_groups[] = $group;
      $bundles++;
      continue;
    }
    $chunks = array_chunk($group['items'], $target_files);
    foreach ($chunks as $chunk) {
      $group['items'] = $chunk;
      $final_groups[] = $group;
      $bundles++;
    }
  }

  $groups = $final_groups;
}
